package Data::Cache::Memory;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Data::Cache::Memory - Object cache implemented above regular file storage

=head1 SYNOPSIS

 use Data::Cache::Memory;

 # Instantinate data cache in given folder (uses default TTL)
 my $cache = Data::Cache::Memory->new('./spool/cache');

 # Instantinate data cache in given folder with optional TTL (in seconds)
 my $cache = Data::Cache::Memory->new('./spool/cache', 60*60*24*7);

 # Instantinate data cache in given folder with optional TTL (in seconds).
 # Automatically perform cache cleanup
 my $cache = Data::Cache::Memory->new('./spool/cache', 60*60*24*7, 1);

 my $testa = ['array1', 'test', 'values'];
 my $testb = {a => '1', b => '1', c => '3'};

 # Store objects into cache
 my $errora = $cache->cache('key1', $testa);
 my $errorb = $cache->cache('key2', $testb);

 # Retrieve objects from cache
 my $test1 = $cache->fetch('key1');
 my $test2 = $cache->fetch('key2');

 # Clear all items from cache with TTL set up during instantination
 my $result = $cache->clear();
 # Clear all items from cache
 my $result = $cache->clear(0);
 # Clear all items from cache older than given TTL (in seconds)
 my $result = $cache->clear(60*60);

=head1 DESCRIPTION

Extensive module description

=head1 AUTHOR

Jan Mach
Cesnet, z.s.p.o
jan.mach@cesnet.cz
http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.

=head1 SEE ALSO

perl(1), Data::Cache(3).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';
use Cwd;
use Storable;
use File::Find;

#use Data::Dumper; #-+-> DEVEL ONLY <-+-#
#use Smart::Comments; #-+-> DEVEL ONLY <-+-#

#-- Perl CPAN modules ---------------------------------------------------------#

#-- Custom application modules ------------------------------------------------#
use Data::Cache;
use Value::Convertor;

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#
use constant TTL_MINUTE => 60;
use constant TTL_HOUR   => TTL_MINUTE() * 60;
use constant TTL_DAY    => TTL_HOUR() * 24;
use constant TTL_WEEK   => TTL_DAY() * 7;

use constant DFLT_TTL   => TTL_HOUR() * 12;

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL @ISA);
    $VERSION = '0.1';
    $DEVEL   = 0;
    @ISA     = ('Data::Cache');
}


#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

=item DESTROY() [PUBLIC]

  Usage     : This method must not be used directly
  Purpose   : Class destructor
  Arguments : None
  Returns   : Nothing
  Throws    : Dies, if not invoked on instance

=cut

sub DESTROY()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    ### Destroying Data.Cache.Folder instance...
}

=item clear(;$) [PUBLIC]

  Usage     : my $object = $instance->clear($timestamp)
  Purpose   : Clear data cache
  Arguments : INTEGER $ttl - time to live in seconds, clear everyting older [OPTIONAL]
  Returns   : Nothing
  Throws    : Dies, if not invoked on instance or in case of error

=cut

sub clear(;$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($ttl,) = @_;
    $ttl = (defined($ttl))?$ttl:$self->{TTL};

    my $ct  = time();  # Get current time
    my $cnt = 0;       # Number of removed items

    foreach my $key (sort keys(%{$self->{CACHE}})) {
        my ($item, $ts) = @{$self->{CACHE}->{$key}};
        if (($ct - $ts) > $ttl) {
            $cnt++;
            delete $self->{CACHE}->{$key};
        }
    }
    return "Deleted $cnt cache objects older than " . Value::Convertor->duration_to_string($ttl);
}

=item fetch($;$) [PUBLIC]

  Usage     : my $object = $instance->fetch($key);
  Purpose   : Fetch data object with given key from cache
  Arguments : STRING $key - object unique key [MANDATORY]
              INTEGER $age - maximal age [OPTIONAL]
  Returns   : MIXED $object on SUCCESS, undef on FAILURE
  Throws    : Dies, if not invoked on instance or given invalid arguments

=cut

sub fetch($;$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($key, $age) = @_;
    confess "Cache key must be given as first argument" unless $key;

    unless (exists $self->{CACHE}->{$key}) {
        $self->{STATS}->{'miss'}++;
        return undef;
    }

    my ($item, $ts) = @{$self->{CACHE}->{$key}};
    my $ct = time;
    if (($age and ($ct - $ts) > $age)) {
        $self->{STATS}->{'aged'}++;
        return undef;
    }

    $self->{STATS}->{'hit'}++;
    return $item;
}

=item cache($$;$) [PUBLIC]

  Usage     : my $object = $instance->cache($key, $object, $overwrite)
  Purpose   : Store data object under given key to cache
  Arguments : STRING $key - object unique key [MANDATORY]
              MIXED $object - object to be stored [MANDATORY]
              BOOL $overwrite - overwrite existing key [OPTIONAL]
  Returns   : STRING $error on FAILURE, EMPTY STRING on SUCCESS
  Throws    : Dies, if not invoked on instance or given invalid arguments

=cut

sub cache($$;$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($key, $object, $overwrite) = @_;
    confess "Cache key must be given as first argument" unless $key;

    # Overwrite must be explicitly requested
    if (exists $self->{CACHE}->{$key}) {
        unless ($overwrite) {
            return "Cache item for key '$key' already exists";
        }
    }

    # Store the object to the cache
    $self->{STATS}->{'cached'}++;
    $self->{CACHE}->{$key} = [$object, time];

    return undef;
}


#-------------------------------------------------------------------------------
# Private interface
#-------------------------------------------------------------------------------

# _init [PROTECTED]
#
#  Usage     : return $self->_init(@_);
#  Purpose   : Initialize the new Data::Cache::Memory instance
#  Arguments : STRING $cache_folder - name of the cache folder [MANDATORY]
#              INTEGER $ttl - TTL of cache items [OPTIONAL]
#              BOOL $clear - clear old items from cache upon startup [OPTIONAL]
#  Returns   : Data::Cache::Memory $self reference
#  Throws    : Dies, if not invoked on instance or given invalid arguments
#  Comments  : This method must never be called directly, it is supposed to be overloaded by subclasses.

sub _init
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    ### Initializing Data.Cache.Folder instance...
    my ($ttl, $clear) = @_;
    confess "Cache TTL must be integer and greater than zero" if $ttl and not int($ttl);

    # Setup internal configurations
    $self->{CACHE} = {};
    $self->{STATS} = {};
    $self->{TTL}   = ($ttl)?int($ttl):DFLT_TTL();

    # Clear old values from cache
    $self->clear() if $clear;

    return $self;
}

=pod

=back

=cut

1;
