package Data::Core;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Data::Core - String related utilities

=head1 SYNOPSIS

 # Import general constants into local namespace
 use Data::Core qw(:c_general);
 # Import logical operation constants into local namespace
 use Data::Core qw(:c_logop);
 # Import modification operation constants into local namespace
 use Data::Core qw(:c_modop);
 # Import all operation constants into local namespace
 use Data::Core qw(:c_opers);
 # Import data type constants into local namespace
 use Data::Core qw(:c_dtypes);

=head1 DESCRIPTION



=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.


=head1 SEE ALSO

perl(1).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;

#use Data::Dumper; #--- DEVELOPMENT ONLY ---#
#use Smart::Comments; #--- DEVELOPMENT ONLY ---#

#-- Perl CPAN modules ---------------------------------------------------------#

#-- Custom application modules ------------------------------------------------#

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#

#-------------------------------------------------------------------------------
# GENERAL CONSTANTS
#-------------------------------------------------------------------------------
use constant TRUE    => 1;
use constant FALSE   => 0;
use constant YES     => 1;
use constant NO      => 0;
use constant SUCCESS => 0;
use constant FAILURE => 1;

#-------------------------------------------------------------------------------
# OPERATIONS
#-------------------------------------------------------------------------------
use constant OP_AND => 'AND';
use constant OP_OR  => 'OR';
use constant OP_XOR => 'XOR';
use constant OP_NOT => 'NOT';

use constant OP_EQ   => 'EQ';
use constant OP_NE   => 'NE';
use constant OP_GT   => 'GT';
use constant OP_LT   => 'LT';
use constant OP_GE   => 'GE';
use constant OP_LE   => 'LE';
use constant OP_IN   => 'IN';
use constant OP_LIKE => 'LIKE';

use constant OP_ADD => 'ADD';
use constant OP_SUB => 'SUB';
use constant OP_MUL => 'MUL';
use constant OP_DIV => 'DIV';
use constant OP_MOD => 'MOD';
use constant OP_CON => 'CON';
use constant OP_REP => 'REP';

#-------------------------------------------------------------------------------
# LOW-LEVEL DATATYPES
#
# !!!! WARNING !!! Some of these datatypes are hardcoded in filter grammar !!!!
#-------------------------------------------------------------------------------
use constant DT_UNKNOWN   => 'UNKNOWN';
use constant DT_DEPENDENT => 'DEPENDENT';
use constant DT_VARIABLE  => 'VARIABLE';
use constant DT_BOOL      => 'BOOL';
use constant DT_BYTE      => 'BYTE';
use constant DT_DATETIME  => 'DATETIME';
use constant DT_DATE      => 'DATE';
use constant DT_TIME      => 'TIME';
use constant DT_ENUM      => 'ENUM';
use constant DT_INT       => 'INT';
use constant DT_NTPSTAMP  => 'NTPSTAMP';
use constant DT_PORTLIST  => 'PORTLIST';
use constant DT_REAL      => 'REAL';
use constant DT_STRING    => 'STRING';

#-------------------------------------------------------------------------------
# HIGH-LEVEL DATATYPES
#-------------------------------------------------------------------------------
use constant DT_PERIOD   => 'PERIOD';
use constant DT_NUMERIC  => 'NUMERIC';
use constant DT_ADDRESS  => 'ADDRESS';
use constant DT_IPV4     => 'IPV4';
use constant DT_IPV6     => 'IPV6';
use constant DT_MAC      => 'MAC';
use constant DT_EMAIL    => 'EMAIL';
use constant DT_DURATION => 'DURATION';
use constant DT_REGEXP   => 'REGEXP';

use constant DT_IPV4_STR          => 'IPV4-ADDR';
use constant DT_IPV4_HEX          => 'IPV4-ADDR-HEX';
use constant DT_IPV4_STR_NET      => 'IPV4-NET';
use constant DT_IPV4_STR_NET_MASK => 'IPV4-NET-MASK';
use constant DT_IPV6_STR          => 'IPV6-ADDR';
use constant DT_IPV6_HEX          => 'IPV6-ADDR-HEX';
use constant DT_IPV6_STR_NET      => 'IPV6-NET';
use constant DT_IPV6_STR_NET_MASK => 'IPV6-NET-MASK';

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use Exporter ();

    our ($VERSION, @ISA, @EXPORT, @EXPORT_OK, %EXPORT_TAGS);

    $VERSION = 1.00;
    @ISA = qw(Exporter);

    # Autoexport - currently unused
    #@EXPORT = qw(&func &func);

    # Export by tags
    %EXPORT_TAGS = (
        c_general => [qw(TRUE FALSE YES NO SUCCESS FAILURE)],
        c_logop   => [qw(OP_AND OP_OR OP_XOR OP_NOT OP_EQ OP_NE OP_GT OP_LT OP_GE OP_LE OP_IN OP_LIKE)],
        c_modop   => [qw(OP_ADD OP_SUB OP_MUL OP_DIV OP_MOD OP_CON OP_REP)],
        c_opers   => [qw(OP_AND OP_OR OP_XOR OP_EQ OP_NE OP_GT OP_LT OP_GE OP_LE OP_IN OP_LIKE
                         OP_ADD OP_SUB OP_MUL OP_DIV OP_MOD OP_CON OP_REP)],
        c_dtypes  => [qw(DT_UNKNOWN DT_DEPENDENT DT_VARIABLE DT_BOOL DT_BYTE DT_DATETIME DT_DATE DT_TIME
                         DT_ENUM DT_INT DT_NTPSTAMP DT_PORTLIST DT_REAL DT_STRING
                         DT_PERIOD DT_NUMERIC DT_ADDRESS DT_IPV4 DT_IPV6 DT_MAC
                         DT_EMAIL DT_DURATION DT_IPV4_STR DT_IPV4_HEX DT_IPV4_STR_NET
                         DT_IPV4_STR_NET_MASK DT_IPV6_STR DT_IPV6_HEX DT_IPV6_STR_NET
                         DT_IPV6_STR_NET_MASK DT_REGEXP)],
    );

    # Everything exported must be in this array
    @EXPORT_OK = ();
    foreach my $tag (keys %EXPORT_TAGS) {
        push(@EXPORT_OK, @{$EXPORT_TAGS{$tag}});
    }

    # Some custom export groups
    $EXPORT_TAGS{all} = [@EXPORT_OK];
}

#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

# No implementation in this module

=pod

=back

=cut

1;
