package Data::Path::Value;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Data::Path::Value - Utility class for determining datatypes of arbitrary object path values

=head1 SYNOPSIS

 use Data::Path::Value;

=head1 DESCRIPTION


=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.

=head1 SEE ALSO

perl(1).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';

use Data::Dumper; #-+-> DEVEL ONLY <-+-#
#use Smart::Comments; #--- DEVELOPMENT ONLY ---#

#-- Perl CPAN modules ---------------------------------------------------------#
use JSON;

#-- Custom application modules ------------------------------------------------#
use Data::Core qw(:c_dtypes);

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL);
    $VERSION = '0.1';
    $DEVEL   = 0;
}


#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

=item new() [PUBLIC,STATIC]

  Usage     : my $instance = Data::Filter::Traverser->new()
  Purpose   : Create and return reference to the new instance
  Arguments : Same as the the _init() method
  Returns   : Data::Filter::Traverser reference on success, croaks on failure
  Throws    : Croaks, if invoked on object
  Comments  : Internally uses _init() method, subclasses should never overload this method.
  See Also  : _init() method

=cut

sub new
{
    my $class = shift;
    confess "Class method not invoked on class" if blessed($class);
    my %args = @_;

    my $self = bless ({}, $class);

    $self->{STRUCTURE} = {};
    if ($args{structure}) {
        my $s = $args{structure};
        unless (ref $s eq 'HASH') {
            $self->_load($s);
        }
        $self->{STRUCTURE} = $s;
    }

    return $self->_init(@_);
}

=item path_datatype($) [PUBLIC,STATIC]

  Usage     : my $datatype = Data::Path::Value->path_datatype($mpath)
  Purpose   : Determine the data type of the given MPath node
  Arguments : STRING containing MPath
  Returns   :
  Throws    : Croaks, if invoked on object

=cut

sub path_datatype($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($dpath,) = @_;
    confess "Data path must be given as argument" unless $dpath;
    return (exists $self->{STRUCTURE}->{$dpath}) ? $self->{STRUCTURE}->{$dpath} : DT_UNKNOWN;
}

#-------------------------------------------------------------------------------
# Private interface
#-------------------------------------------------------------------------------

# _init($) [PROTECTED]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Croaks, if invoked on class

sub _init($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    return $self;
}

# _load($) [PROTECTED]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Croaks, if invoked on class

sub _load($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($file,) = @_;
    confess "Data structure file '$file' is not ordinary file" unless -f $file;

    # Load message from external file to string and untaint it
    open (my $f, $file) or die "Can`t open data structure file '$file'\n";
    my $json;
    while (my $line = <$f>) {
        # Custom feature - remove comments
        next if $line =~ /^\s*(#|;|\/\/)/;
        $json .= $line;
    }
    close($f);

    # Untaint the loaded string
    if ($json =~ /^(.+)$/s) { $json = $1; }

    # Parse out the JSON object and return it
    return decode_json($json);
}

=pod

=back

=cut

1;
