package Mentat::App;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Mentat::App - String related utilities

=head1 SYNOPSIS

 use Mentat::App qw(:const);

=head1 DESCRIPTION



=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.


=head1 SEE ALSO

perl(1).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Cwd qw(abs_path);
use Pod::Usage;

#use Data::Dumper; #--- DEVELOPMENT ONLY ---#
#use Smart::Comments; #--- DEVELOPMENT ONLY ---#

#-- Perl CPAN modules ---------------------------------------------------------#

#-- Custom application modules ------------------------------------------------#
use Log::Loger;
use App::Env;
use App::Env::Scanner;
use Config::Validator;
use Mentat::Core qw(:c_processor :func);

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#

# List of all configurations
my %CONFIGS;

# Configurations for the environment
my $DEFAULT_ENVIRONMENT_CONFIG;

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use Exporter ();

    our ($VERSION, @ISA, @EXPORT, @EXPORT_OK, %EXPORT_TAGS);

    $VERSION = 1.00;
    @ISA = qw(Exporter);

    # Autoexport - currently unused
    #@EXPORT = qw(&func &func);

    # Export by tags
    #%EXPORT_TAGS = (tag => [qw(key)]);

    # Everything exported must be in this array
    #@EXPORT_OK = ();
    #foreach my $tag (keys %EXPORT_TAGS) {
    #   push(@EXPORT_OK, @{$EXPORT_TAGS{$tag}});
    #}

    # Some custom export groups
    #$EXPORT_TAGS{all} = [@EXPORT_OK];

    #---------------------------------------------------------------------------
    # ADDITIONAL CONFIGURATIONS
    #---------------------------------------------------------------------------
    %CONFIGS = (
        #-----------------------------------------------------------------------
        # COMPONENT DESCRIPTION CONFIGURATIONS
        #-----------------------------------------------------------------------
        id => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'id',
            App::Env::KW_TYPE()        => 'string',
            App::Env::KW_DEFAULT()     => App::Env::Scanner::host_name() . '-component',
            App::Env::KW_DESCRIPTION() => 'Unique identifier of this component within the detection environment',
            App::Env::KW_REQUIRED()    => 1,
        },
        host_name => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'host-name',
            App::Env::KW_TYPE()        => 'hostname',
            App::Env::KW_DEFAULT()     => App::Env::Scanner::host_name(),
            App::Env::KW_DESCRIPTION() => 'DNS host name of the device running this component',
        },
        #-----------------------------------------------------------------------
        # COMMON COMPONENT CONFIGURATIONS
        #-----------------------------------------------------------------------
        runlog_file => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'runlog-file',
            App::Env::KW_TYPE()        => 'file',
            App::Env::KW_DEFAULT()     => p(loc_run."/".app_name.".rl"),
            App::Env::KW_DESCRIPTION() => 'Name of the file to which to dump runlog',
            App::Env::KW_REQUIRED()    => 1,
        },
        stats_file => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'stats-file',
            App::Env::KW_TYPE()        => 'file',
            App::Env::KW_DEFAULT()     => p(loc_run."/".app_name.".stats"),
            App::Env::KW_DESCRIPTION() => 'Name of the file to which to dump statistics',
            App::Env::KW_REQUIRED()    => 1,
        },
        dump_file => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'dump-file',
            App::Env::KW_TYPE()        => 'file',
            App::Env::KW_DEFAULT()     => p(loc_run."/".app_name.".dump"),
            App::Env::KW_DESCRIPTION() => 'Name of the file to which to dump everything',
            App::Env::KW_REQUIRED()    => 1,
        },
        lock_file => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'lock-file',
            App::Env::KW_TYPE()        => 'file',
            App::Env::KW_DEFAULT()     => p(loc_run."/".app_name.".lock"),
            App::Env::KW_DESCRIPTION() => 'Name of the lock file for mutual exclusion',
            App::Env::KW_REQUIRED()    => 1,
        },
        pid_file => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'pid-file',
            App::Env::KW_TYPE()        => 'file',
            App::Env::KW_DEFAULT()     => p(loc_run."/".app_name.".pid"),
            App::Env::KW_DESCRIPTION() => 'Name of the daemon PID file',
            App::Env::KW_REQUIRED()    => 1,
        },
        user => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'user',
            App::Env::KW_TYPE()        => 'string',
            App::Env::KW_DEFAULT()     => 'mentat',
            App::Env::KW_DESCRIPTION() => 'User name under which to run the application',
            App::Env::KW_REQUIRED()    => 1,
        },
        group => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'group',
            App::Env::KW_TYPE()        => 'string',
            App::Env::KW_DEFAULT()     => 'mentat',
            App::Env::KW_DESCRIPTION() => 'Group name under which to run the application',
            App::Env::KW_REQUIRED()    => 1,
        },
        in_foreground => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'in-foreground',
            App::Env::KW_TYPE()        => 'bool',
            App::Env::KW_DEFAULT()     => 0,
            App::Env::KW_DESCRIPTION() => 'Let the application to stay in foreground',
        },
        log => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'log',
            App::Env::KW_TYPE()        => 'string',
            App::Env::KW_STACK()       => 1,
            App::Env::KW_DEFAULT()     => [
                    "std:err:info",
                    #'syslog::info',
                    #'email:mentat@domain.org;info@domain.org;Mentat - Alert message:emerg',
                    #"file:".p(loc_log()."/".app_name.".log").":info",
                ],
            App::Env::KW_DESCRIPTION() => 'Defined log channels',
            App::Env::KW_REQUIRED()    => 1,
        },
        #-----------------------------------------------------------------------
        # VALIDATOR ENGINE CONFIGURATIONS
        #-----------------------------------------------------------------------
        schema_file_idea => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'schema-file-idea',
            App::Env::KW_TYPE()        => 'file:e,s,f,r',
            App::Env::KW_DEFAULT()     => '/usr/share/idea/idea.schema',
            App::Env::KW_DESCRIPTION() => 'Name of the file containing IDEA JSON schema',
            App::Env::KW_REQUIRED()    => 1,
        },
        schema_file => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'schema-file',
            App::Env::KW_TYPE()        => 'file:e,s,f,r',
            App::Env::KW_DEFAULT()     => '/usr/share/idea/idea.schema',
            App::Env::KW_DESCRIPTION() => 'Name of the file containing message schema',
            App::Env::KW_REQUIRED()    => 0,
        },
        template_schema_file => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'template-schema-file',
            App::Env::KW_TYPE()        => 'file:e,s,f,r',
            App::Env::KW_DESCRIPTION() => 'Name of the file containing message template schema',
            App::Env::KW_REQUIRED()    => 0,
        },
        #-----------------------------------------------------------------------
        # FILTER ENGINE CONFIGURATIONS
        #-----------------------------------------------------------------------
        filter_grammar_file => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'filter-grammar-file',
            App::Env::KW_TYPE()        => 'file:e,s,f,r',
            App::Env::KW_DEFAULT()     => p(loc_etc.'/.core/filter-grammar.conf'),
            App::Env::KW_DESCRIPTION() => 'File containing filter grammar specification',
        },
        filter_schema_file => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'filter-schema-file',
            App::Env::KW_TYPE()        => 'file:e,s,f,r',
            App::Env::KW_DEFAULT()     => p(loc_etc.'/.core/schema-data-filter.json'),
            App::Env::KW_DESCRIPTION() => 'File containing filter schema specification',
        },
        filter_rules_file => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'filter-rules-file',
            App::Env::KW_TYPE()        => 'file:e,s,f,r',
            App::Env::KW_DEFAULT()     => p(loc_etc.'/filter-rules.json'),
            App::Env::KW_DESCRIPTION() => 'File containing message filtering rules',
        },
        filter_rules_file_test => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'filter-rules-file-test',
            App::Env::KW_TYPE()        => 'file:e,s,f,r',
            App::Env::KW_DEFAULT()     => p(loc_etc.'/filter-rules-test.json'),
            App::Env::KW_DESCRIPTION() => 'File containing message filtering rules for test data',
        },
        #-----------------------------------------------------------------------
        # PICKUP AND FILESTORE ENGINE CONFIGURATIONS
        #-----------------------------------------------------------------------
        watched_folder => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'folder',
            App::Env::KW_TYPE()        => 'file:d,w',
            App::Env::KW_DEFAULT()     => p(loc_var.'/spool/incoming'),
            App::Env::KW_DESCRIPTION() => 'Name of the directory to watch',
            App::Env::KW_REQUIRED()    => 1,
        },
        queue_folder => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'queue',
            App::Env::KW_TYPE()        => 'file:d,w',
            App::Env::KW_DEFAULT()     => p(loc_var.'/spool/pending'),
            App::Env::KW_DESCRIPTION() => 'Name of the directory containing processing queue',
            App::Env::KW_REQUIRED()    => 1,
        },
        done_folder => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'done-folder',
            App::Env::KW_TYPE()        => 'file:d,w',
            App::Env::KW_DEFAULT()     => p(loc_var.'/spool/done'),
            App::Env::KW_DESCRIPTION() => 'Folder to which to store done alert messages',
        },
        temp_folder => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'temp-folder',
            App::Env::KW_TYPE()        => 'file:d,w',
            App::Env::KW_DEFAULT()     => p(loc_var.'/spool/tmp'),
            App::Env::KW_DESCRIPTION() => 'Folder to be used as temporary folder',
            App::Env::KW_REQUIRED()    => 1,
        },
        error_folder => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'error-folder',
            App::Env::KW_TYPE()        => 'file:d,w',
            App::Env::KW_DEFAULT()     => p(loc_var.'/spool/errors'),
            App::Env::KW_DESCRIPTION() => 'Folder to which to store invalid alert messages',
            App::Env::KW_REQUIRED()    => 1,
        },
        pickup_limit => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'pickup-limit',
            App::Env::KW_TYPE()        => 'int',
            App::Env::KW_DEFAULT()     => 5,
            App::Env::KW_DESCRIPTION() => 'Maximum number of messages to pickup from the folder at a time',
            App::Env::KW_REQUIRED()    => 1,
        },
        one_shot => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'one-shot',
            App::Env::KW_TYPE()        => 'bool',
            App::Env::KW_DEFAULT()     => 0,
            App::Env::KW_DESCRIPTION() => 'Stop processing when everything is processed (do not wait for more data)',
            App::Env::KW_REQUIRED()    => 1,
        },
        #-----------------------------------------------------------------------
        store_folder => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'store-folder',
            App::Env::KW_TYPE()        => 'file:d,w',
            App::Env::KW_DEFAULT()     => p(loc_var.'/spool/incoming'),
            App::Env::KW_DESCRIPTION() => 'Folder to which to store incoming alert messages',
            App::Env::KW_REQUIRED()    => 1,
        },
        #-----------------------------------------------------------------------
        # STORAGE ENGINE CONFIGURATIONS
        #-----------------------------------------------------------------------
        database => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'database',
            App::Env::KW_TYPE()        => 'string',
            App::Env::KW_DEFAULT()     => 'mongo',
            App::Env::KW_DESCRIPTION() => 'Specification of the database used for the permanent storage',
            App::Env::KW_REQUIRED()    => 1,
        },
        #-----------------------------------------------------------------------
        # TRAGGREGATOR AND CORRELATOR ENGINE CONFIGURATIONS
        #-----------------------------------------------------------------------
        traggregator_rules_file => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'traggregator-rules-file',
            App::Env::KW_TYPE()        => 'file:e,s,f,r',
            App::Env::KW_DEFAULT()     => p(loc_etc.'/traggregation-rules.conf'),
            App::Env::KW_DESCRIPTION() => 'File containing message threshodling and aggregation rules',
        },
        #-----------------------------------------------------------------------
        # CRUNCHER ENGINE CONFIGURATIONS
        #-----------------------------------------------------------------------
        state_storage => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'state-storage',
            App::Env::KW_TYPE()        => 'string',
            App::Env::KW_DEFAULT()     => p(loc_run()."/".app_name.".state"),
            App::Env::KW_DESCRIPTION() => 'Name of the file for persistent state storage',
            App::Env::KW_REQUIRED()    => 1,
        },
        cron => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'cron',
            App::Env::KW_TYPE()        => 'bool',
            App::Env::KW_DEFAULT()     => 0,
            App::Env::KW_DESCRIPTION() => 'Running as cron job',
        },
        run_interval => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'run-interval',
            App::Env::KW_TYPE()        => 'int',
            App::Env::KW_DEFAULT()     => 300,
            App::Env::KW_DESCRIPTION() => 'Run interval in seconds',
            App::Env::KW_REQUIRED()    => 1,
        },
        adjust_time_boundaries => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'adjust-time-boundaries',
            App::Env::KW_TYPE()        => 'enum:no,mod,2hrs,daily',
            App::Env::KW_DEFAULT()     => 'mod',
            App::Env::KW_DESCRIPTION() => 'Adjust time boundaries',
        },
        time_high => {
            App::Env::KW_OPTION()      => 'time-high',
            App::Env::KW_TYPE()        => 'int',
            App::Env::KW_DEFAULT()     => 0,
            App::Env::KW_DESCRIPTION() => 'High time boundary',
            App::Env::KW_REQUIRED()    => 1,
        },
        reports_dir => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'reports-dir',
            App::Env::KW_TYPE()        => 'string',
            App::Env::KW_DEFAULT()     => p(loc_var()."/reports"),
            App::Env::KW_DESCRIPTION() => 'Name of the directory to store generated reports into',
            App::Env::KW_REQUIRED()    => 1,
        },
        charts_dir => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'charts-dir',
            App::Env::KW_TYPE()        => 'string',
            App::Env::KW_DEFAULT()     => p(loc_var()."/charts"),
            App::Env::KW_DESCRIPTION() => 'Name of the directory to store generated charts into',
            App::Env::KW_REQUIRED()    => 1,
        },
        #-----------------------------------------------------------------------
        # STATISTICIAN CRUNCHER ENGINE CONFIGURATIONS
        #-----------------------------------------------------------------------
        generate_only => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'generate-only',
            App::Env::KW_TYPE()        => 'bool',
            App::Env::KW_DEFAULT()     => 0,
            App::Env::KW_DESCRIPTION() => 'Only generate graphs',
        },
        rrds_dir => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'rrds-dir',
            App::Env::KW_TYPE()        => 'string',
            App::Env::KW_DEFAULT()     => p(loc_var()."/rrds"),
            App::Env::KW_DESCRIPTION() => 'Name of the directory to store RRD databases into',
            App::Env::KW_REQUIRED()    => 1,
        },
        colors => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_TYPE()        =>  'hash',
            App::Env::KW_DEFAULT()     => {'_master' => 'red'},
            App::Env::KW_DESCRIPTION() => 'Color definitions for data sources',
        },
        #-----------------------------------------------------------------------
        # REPORTER CRUNCHER ENGINE CONFIGURATIONS
        #-----------------------------------------------------------------------
        whois_dump_file => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'whois-dump-file',
            App::Env::KW_TYPE()        => 'string',
            App::Env::KW_DEFAULT()     => p(loc_var()."/whois-negistry.json"),
            App::Env::KW_DESCRIPTION() => 'Name of the file containing static whois dump',
            App::Env::KW_REQUIRED()    => 1,
        },
        mail_to => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'mail-to',
            App::Env::KW_TYPE()        => 'string',
            App::Env::KW_DESCRIPTION() => 'Email address to which to send the report',
            App::Env::KW_REQUIRED()    => 1,
        },
        mail_from => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'mail-from',
            App::Env::KW_TYPE()        => 'string',
            App::Env::KW_DESCRIPTION() => 'Email address from which the reports come from',
            App::Env::KW_REQUIRED()    => 1,
        },
        mail_admin => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'mail-admin',
            App::Env::KW_TYPE()        => 'string',
            App::Env::KW_DESCRIPTION() => 'Email address for system administrators',
            App::Env::KW_REQUIRED()    => 1,
        },
        mail_devel => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'mail-devel',
            App::Env::KW_TYPE()        => 'string',
            App::Env::KW_DESCRIPTION() => 'Email address for system developers',
            App::Env::KW_REQUIRED()    => 1,
        },
        mail_test => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'mail-test',
            App::Env::KW_TYPE()        => 'string',
            App::Env::KW_DESCRIPTION() => 'Email address for test data recipients',
            App::Env::KW_REQUIRED()    => 1,
        },
        mail_bcc => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'mail-bcc',
            App::Env::KW_TYPE()        => 'string',
            App::Env::KW_DESCRIPTION() => 'Email address to which to send the BCC of the report',
            App::Env::KW_REQUIRED()    => 0,
        },
        reply_to => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'reply-to',
            App::Env::KW_TYPE()        => 'string',
            App::Env::KW_DESCRIPTION() => 'Reply-To email address for reports',
            App::Env::KW_REQUIRED()    => 1,
        },
        template_dir => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'template-dir',
            App::Env::KW_TYPE()        => 'string',
            App::Env::KW_DEFAULT()     => p(loc_etc()."/formater-templates"),
            App::Env::KW_DESCRIPTION() => 'Name of the directory containing templates for email formater',
            App::Env::KW_REQUIRED()    => 1,
        },
        exceptions_file => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'exceptions-file',
            App::Env::KW_TYPE()        => 'string',
            App::Env::KW_DEFAULT()     => p(loc_etc()."/reporter-exceptions.conf"),
            App::Env::KW_DESCRIPTION() => 'Name of the file containing reporting exceptions',
        },
        resolve => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'resolve',
            App::Env::KW_TYPE()        => 'bool',
            App::Env::KW_DEFAULT()     => 1,
            App::Env::KW_DESCRIPTION() => 'Switch for IP address resolving',
            App::Env::KW_REQUIRED()    => 1,
        },
        testmode => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'testmode',
            App::Env::KW_TYPE()        => 'bool',
            App::Env::KW_DEFAULT()     => 0,
            App::Env::KW_DESCRIPTION() => 'Switch for test mode',
            App::Env::KW_REQUIRED()    => 1,
        },
        testdata => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'testdata',
            App::Env::KW_TYPE()        => 'bool',
            App::Env::KW_DEFAULT()     => 0,
            App::Env::KW_DESCRIPTION() => 'Switch for processing test data',
            App::Env::KW_REQUIRED()    => 1,
        },
        #-----------------------------------------------------------------------
        # REPORTER-NG CONFIGURATIONS
        #-----------------------------------------------------------------------
        abuse => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'abuse',
            App::Env::KW_TYPE()        => 'string',
            App::Env::KW_DESCRIPTION() => 'Name of the abuse for which to generate the report',
        },
        period => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'period',
            App::Env::KW_TYPE()        => 'string',
            App::Env::KW_DESCRIPTION() => 'Time period for which to generate the onetime report',
        },
        #-----------------------------------------------------------------------
        # BRIEFER CONFIGURATIONS
        #-----------------------------------------------------------------------
        brief_date => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'brief-date',
            App::Env::KW_TYPE()        => 'string',
            App::Env::KW_DESCRIPTION() => 'Brief date',
        },
        brief_type => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'brief-type',
            App::Env::KW_TYPE()        => 'enum:day,week,month',
            App::Env::KW_DEFAULT()     => 'day',
            App::Env::KW_DESCRIPTION() => 'Brief type',
            App::Env::KW_REQUIRED()    => 1,
        },
        #-----------------------------------------------------------------------
        # HANDYMAN CONFIGURATIONS
        #-----------------------------------------------------------------------
        node_name => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'node-name',
            App::Env::KW_TYPE()        => 'string',
            App::Env::KW_DESCRIPTION() => 'Unique name of the current node, from which the message originates',
        },
        node_type => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'node-type',
            App::Env::KW_TYPE()        => 'string',
            App::Env::KW_DESCRIPTION() => 'Type of the current node, from which the message originates',
        },
        node_sw => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'node-sw',
            App::Env::KW_TYPE()        => 'string',
            App::Env::KW_DESCRIPTION() => 'Detection SW on the current node, from which the message originates',
        },
        node_note => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'node-note',
            App::Env::KW_TYPE()        => 'string',
            App::Env::KW_DESCRIPTION() => 'Note for the current node, from which the message originates',
        },
        detect_time => {
            App::Env::KW_OPTION()      => 'detect-time',
            App::Env::KW_TYPE()        => 'int',
            App::Env::KW_DESCRIPTION() => 'UNIX timestamp of the message detection time',
        },
        pickup_folder => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'pickup-folder',
            App::Env::KW_TYPE()        => 'file:d,w',
            App::Env::KW_DEFAULT()     => p(loc_var()."/spool/incoming"),
            App::Env::KW_DESCRIPTION() => 'Name of the pickup folder to which to put the generated message',
        },
        cache_folder => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'cache-folder',
            App::Env::KW_TYPE()        => 'file:d,w',
            App::Env::KW_DEFAULT()     => p(loc_var()."/cache"),
            App::Env::KW_DESCRIPTION() => 'Name of the cache folder',
        },
        message_type => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'message-type',
            App::Env::KW_TYPE()        => 'enum:idea',
            App::Env::KW_DEFAULT()     => 'idea',
            App::Env::KW_DESCRIPTION() => 'Type of the generated messsage',
            App::Env::KW_REQUIRED()    => 1,
        },
        template_file => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'template-file',
            App::Env::KW_TYPE()        => 'file:f',
            App::Env::KW_DESCRIPTION() => 'Name of the file containing Mentat message generation templates',
            App::Env::KW_REQUIRED()    => 1,
        },
        template => {
            App::Env::KW_OPTION()      => 'template',
            App::Env::KW_TYPE()        => 'string',
            App::Env::KW_DESCRIPTION() => 'Identifier of the template in the template file, which to use to generate message',
            App::Env::KW_REQUIRED()    => 0,
        },
        template_list => {
            App::Env::KW_OPTION()      => 'template-list',
            App::Env::KW_TYPE()        => 'bool',
            App::Env::KW_DEFAULT()     => 0,
            App::Env::KW_DESCRIPTION() => 'Dump list of all available templates',
        },
        stdout => {
            App::Env::KW_OPTION()      => 'stdout',
            App::Env::KW_TYPE()        => 'bool',
            App::Env::KW_DEFAULT()     => 0,
            App::Env::KW_DESCRIPTION() => 'Generate the message to the standard output instead of file',
        },
        test => {
            App::Env::KW_OPTION()      => 'test',
            App::Env::KW_TYPE()        => 'bool',
            App::Env::KW_DEFAULT()     => 0,
            App::Env::KW_DESCRIPTION() => 'Generate the example messages in given template file',
        },
        test_connection => {
            App::Env::KW_OPTION()      => 'test-connection',
            App::Env::KW_TYPE()        => 'bool',
            App::Env::KW_DEFAULT()     => 0,
            App::Env::KW_DESCRIPTION() => 'Generate the connection test message',
        },
        parade => {
            App::Env::KW_OPTION()      => 'parade',
            App::Env::KW_TYPE()        => 'bool',
            App::Env::KW_DEFAULT()     => 0,
            App::Env::KW_DESCRIPTION() => 'Generate the example message for all configured templates',
        },
        verbose => {
            App::Env::KW_OPTION()      => 'verbose',
            App::Env::KW_TYPE()        => 'bool',
            App::Env::KW_DEFAULT()     => 0,
            App::Env::KW_DESCRIPTION() => 'Verbose output switch',
        },
        #-----------------------------------------------------------------------
        # SCRIPT CONFIGURATIONS
        #-----------------------------------------------------------------------
        plugin => {
            App::Env::KW_OPTION()      => 'plugin',
            App::Env::KW_TYPE()        => 'bool',
            App::Env::KW_DEFAULT()     => 0,
            App::Env::KW_DESCRIPTION() => 'Run script as plugin',
            App::Env::KW_REQUIRED()    => 1,
        },
        store_log => {
            App::Env::KW_OPTION()      => 'store-log',
            App::Env::KW_TYPE()        => 'bool',
            App::Env::KW_DEFAULT()     => 0,
            App::Env::KW_DESCRIPTION() => 'Store script processing log into database',
            App::Env::KW_REQUIRED()    => 1,
        },
        dump_log => {
            App::Env::KW_OPTION()      => 'dump-log',
            App::Env::KW_TYPE()        => 'bool',
            App::Env::KW_DEFAULT()     => 0,
            App::Env::KW_DESCRIPTION() => 'Dump script processing log into file',
            App::Env::KW_REQUIRED()    => 1,
        },
        #-----------------------------------------------------------------------
        # HAWAT-CLI CONFIGURATIONS
        #-----------------------------------------------------------------------
        devel => {
            App::Env::KW_OPTION()      => 'devel',
            App::Env::KW_TYPE()        => 'bool',
            App::Env::KW_DEFAULT()     => 0,
            App::Env::KW_DESCRIPTION() => 'Run in devel mode',
            App::Env::KW_REQUIRED()    => 1,
        },
        email_template_file => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'email-template-file',
            App::Env::KW_TYPE()        => 'string',
            App::Env::KW_DEFAULT()     => p(loc_etc()."/formater-templates/brief/_master.tt"),
            App::Env::KW_DESCRIPTION() => 'Name of the file containing briefing template for email reporting',
            App::Env::KW_REQUIRED()    => 1,
        },
        run_folder => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'run-folder',
            App::Env::KW_TYPE()        => 'file:d,w',
            App::Env::KW_DEFAULT()     => p(loc_var()."/run"),
            App::Env::KW_DESCRIPTION() => 'Name of the run folder',
            App::Env::KW_REQUIRED()    => 1,
        },
        spool_folder => {
            App::Env::KW_CONFIG()      => 1,
            App::Env::KW_OPTION()      => 'spool-folder',
            App::Env::KW_TYPE()        => 'file:d,w',
            App::Env::KW_DEFAULT()     => p(loc_var()."/spool"),
            App::Env::KW_DESCRIPTION() => 'Name of the spool folder',
            App::Env::KW_REQUIRED()    => 1,
        },
    );
}

#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

sub init($)
{
    # Setup the application environment
    my $error = App::Env->init(@_);
    pod2usage(-message => $error, -verbose => 1, -exitval => 1, -output => \*STDERR) if $error;
}

sub get_bare_config(@)
{
    my @l = ();
    push(@l, @_) if scalar @_;

    my $cfg = {App::Env::KW_CONFIGS() => {}};
    foreach my $c (@l) {
        die "Requested inclusion of unknown environment configuration '$c'\n" unless exists $CONFIGS{$c};
        $cfg->{App::Env::KW_CONFIGS()}->{$c} = $CONFIGS{$c};
    }
    return $cfg;
}

sub get_base_config(@)
{
    my @l = qw(log);
    push(@l, @_) if scalar @_;

    my $cfg = {App::Env::KW_CONFIGS() => {}, App::Env::KW_CFG_FILE_NAME() => Mentat::Core::detect_config_file()};
    foreach my $c (@l) {
        die "Requested inclusion of unknown environment configuration '$c'\n" unless exists $CONFIGS{$c};
        $cfg->{App::Env::KW_CONFIGS()}->{$c} = $CONFIGS{$c};
    }
    return $cfg;
}

sub get_env_config(@)
{
    my @l = qw(id host_name runlog_file stats_file dump_file lock_file pid_file user group in_foreground);

    return get_base_config(@l, @_);
}

sub get_proc_config()
{
    return {
        CSECTION_LOGER() => {
                CKEY_LOG() => App::Env->log,
            },
        CSECTION_PROCESSOR() => {
                CKEY_RUNLOG() => App::Env->runlog_file,
                CKEY_PATH() => '/bin:/usr/bin:/usr/local/bin',
                #CKEY_SIGNALS() => ['USR1','USR2'],
            },
        CSECTION_SCHEDULER() => {
                CKEY_CLASS() => 'Mentat::Processor::Scheduler::Default',
            },
        CSECTION_DAEMON() => {
                CKEY_PID_FILE() => App::Env->pid_file,
                CKEY_USER()     => App::Env->user,
                CKEY_GROUP()    => App::Env->group,
                CKEY_INFOREG()  => App::Env->in_foreground,
            },
    };
}

sub get_crun_config()
{
    return {
        CSECTION_LOGER() => {
                CKEY_LOG() => App::Env->log,
            },
        CSECTION_PROCESSOR() => {
                CKEY_RUNLOG() => App::Env->runlog_file,
                CKEY_PATH() => '/bin:/usr/bin:/usr/local/bin',
                #CKEY_SIGNALS() => ['USR1','USR2'],
            },
        CSECTION_SCHEDULER() => {
                CKEY_CLASS() => 'Mentat::Processor::Scheduler::Default',
            },

    };
}

=pod

=back

=cut

1;
