package Mentat::Message::Grouper;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Mentat::Message::Grouper - Short module description

=head1 SYNOPSIS

 use Mentat::Message::Grouper;

=head1 DESCRIPTION

Extensive module description

=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.

=head1 SEE ALSO

perl(1), Parent::Module(3).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';

use Data::Dumper; #-+-> DEVEL ONLY <-+-#

#-- Perl CPAN modules ---------------------------------------------------------#

#-- Custom application modules ------------------------------------------------#

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL);
    $VERSION = '0.1';
    $DEVEL   = 0;
}


#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

=item new() [PUBLIC ,STATIC]

  Usage     : my $instance = Mentat::Message::Grouper->new()
  Purpose   : Create and return reference to the new instance
  Arguments : Same as the the _init() method
  Returns   : Mentat::Message::Grouper reference on success, croaks on failure
  Throws    : Croaks, if invoked on object
  Comments  : Internally uses _init() method, subclasses should never overload this method.
  See Also  : _init() method

=cut

sub new
{
    my $class = shift;
    croak ((caller(0))[3] . ": class method invoked on object") if blessed($class);
    my $self = bless ({}, $class);
    return $self->_init(@_);
}

=item group($$) [PUBLIC ,STATIC]

  Usage     : my $result = $instance->group($rule, $messages);
  Purpose   : Group given messages according to the given rule
  Arguments : MIXED $rule - grouping rule, can be either
                 STRING
              ARRAY REFERENCE $messages - Mentat::Messages to be grouped
  Returns   :
  Throws    : Croaks, if invoked on object
  Comments  :
  See Also  :

=cut

sub group($$)
{
    my $self = shift;
    croak ((caller(0))[3] . ": instance method invoked on class") unless blessed($self);
    my ($rule, $messages) = @_;

    # Prepare the storage for result
    my $result = {};

    # Group all given messages into the result storage
    foreach my $msg (@$messages) {
        $self->_group_into($result, $self->_calculate_subkeys($rule, $msg), $msg, $rule);
    }

    return $result;
}

=item group_multi($$) [PUBLIC ,STATIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Croaks, if invoked on object
  Comments  :
  See Also  :

=cut

sub group_multi($$)
{
    my $self = shift;
    croak ((caller(0))[3] . ": instance method invoked on class") unless blessed($self);
    my ($rules, $messages) = @_;

    # Perform the first grouping
    my $result = $self->group(shift @$rules, $messages);

    # If there are more subsequent groupings
    if (scalar @$rules) {
        my $queuea = [$result];
        my $queueb = [];

        # For each subsequent MPath
        foreach my $rule (@$rules) {
            # Find all subhashes in all result (sub)hashes
            while (my $t = pop(@$queuea)) {
                foreach my $key (keys(%$t)) {
                    push(@$queueb, \$t->{$key});
                }
            }

            # Group all messages in all subhashes and replace the previous result with group
            while (my $t = pop(@$queueb)) {
                my $r = $self->group($rule, $$t);
                $$t = $r;

                push(@$queuea, $r);
            }
        }
    }

    return $result;
}

#-------------------------------------------------------------------------------
# Private interface
#-------------------------------------------------------------------------------

# _init [PROTECTED]
#
#  Usage     : return $self->_init(@_);
#  Purpose   : Initialize the new Mentat::Message::Grouper instance
#  Arguments :
#  Returns   : Mentat::Message::Grouper reference
#  Throws    : Croaks, if invoked on class
#  Comments  : This method must never be called directly, it is supposed to be overloaded by subclasses.
#  See Also  :

sub _init
{
    my $self = shift;
    croak ((caller(0))[3] . ": instance method invoked on class") unless blessed($self);
    return $self;
}

# _calculate_subkeys($$) [PROTECTED]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Croaks, if invoked on class
#  Comments  :
#  See Also  :

sub _calculate_subkeys($$)
{
    my $self = shift;
    croak ((caller(0))[3] . ": instance method invoked on class") unless blessed($self);
    my ($rule, $msg) = @_;

    return $msg->path_values($rule) unless ref $rule;
    die "Grouping rule must be either MPath, or array reference\n" unless ref $rule eq 'ARRAY';

    my $method = @$rule[0];
    my @args = @$rule;
    return &$method($msg, @args[1..$#args]) if ref $method eq 'CODE';

    return $self->$method($msg, @$rule);
}


# _group_into($$$$) [PROTECTED]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Croaks, if invoked on class
#  Comments  :
#  See Also  :

sub _group_into($$$$)
{
    my $self = shift;
    croak ((caller(0))[3] . ": instance method invoked on class") unless blessed($self);
    my ($result, $keys, $message, $rule) = @_;

    unless (scalar @$keys) {
        push(@{$result->{''}}, $message);
        return $result;
    }

    foreach my $key (@$keys) {
        push(@{$result->{$key}}, $message);
    }
    return $result;
}

=pod

=back

=cut

1;
