package Mentat::Processor::Queue::Folder;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Mentat::Processor::Queue::Folder - Folder queue implementation

=head1 SYNOPSIS

 use Mentat::Processor::Queue::Folder;

=head1 DESCRIPTION



=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.

=head1 SEE ALSO

perl(1).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';
use Time::HiRes qw(time sleep);
use Digest::MD5 qw(md5_hex);
use DirHandle;
use File::Basename;
use File::Copy;

#use Data::Dumper; #-+-> DEVEL ONLY <-+-#
#use Smart::Comments; #-+-> DEVEL ONLY <-+-#

#-- Perl CPAN modules ---------------------------------------------------------#

#-- Custom application modules ------------------------------------------------#


#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#
use constant FETCH_LIMIT => 10000;

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL);
    $VERSION = '0.1';
    $DEVEL   = 0;
}

#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

=item new() [PUBLIC,STATIC]

  Usage     : my $instance = Mentat::Processor::Queue::Folder;->new()
  Purpose   : Create and return reference to the new instance
  Arguments : Same as the the _init() method
  Returns   : Mentat::Processor::Queue::Folder; reference on success, croaks on failure
  Throws    : Dies, if not invoked on class
  Comments  : Internally uses _init() method, subclasses should never overload this method.
  See Also  : _init() method

=cut

sub new
{
    my $class = shift;
    confess "Class method not invoked on class" if blessed($class);
    ### Creating Mentat.Processor.Queue.Folder instance...
    my $self = bless ({}, $class);
    return $self->_init(@_);
}

=item DESTROY() [PUBLIC]

  Usage     : This method must not be used directly
  Purpose   : Class destructor
  Arguments : None
  Returns   : Nothing
  Throws    : Dies, if not invoked on instance

=cut

sub DESTROY()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    ### Destroying Mentat.Processor.Queue.Folder instance...
}

#-------------------------------------------------------------------------------

=item enqueue($) [PUBLIC]

  Usage     : my $id = $instance->enqueue($message);
  Purpose   : Enqueue new item into the INCOMING message queue
  Arguments : MIXED $item - item to be queued [MANDATORY]
  Returns   : STRING $queue_id identifier of the message within the queue
  Throws    : Dies, if not invoked on object instance or if given invalid arguments

=cut

sub enqueue($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($item,) = @_;
    confess "Queue item must be given as first argument" unless $item;

    return $self->enqueue_msg($item) if blessed($item);
    return $self->enqueue_file($item);
}

=item enqueue_file($) [PUBLIC]

  Usage     : my $id = $instance->enqueue($message);
  Purpose   : Enqueue new message contained in given file into the INCOMING message queue
  Arguments : STRING $file - message to be queued [MANDATORY]
  Returns   : STRING $queue_id identifier of the message within the queue
  Throws    : Dies, if not invoked on object instance or if given invalid arguments

=cut

sub enqueue_file($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($file,) = @_;

    my $result = _move($file, $self->{F_INCOMING});
    $self->statistics->{enqueue_file}++;
    return $result;
}

=item enqueue_msg($) [PUBLIC]

  Usage     : my $id = $instance->enqueue_f($message);
  Purpose   : Enqueue new message into the INCOMING message queue
  Arguments : REFERENCE $message - message to be queued [MANDATORY]
  Returns   : STRING $queue_id identifier of the message within the queue
  Throws    : Dies, if not invoked on object instance or if given invalid arguments

=cut

sub enqueue_msg($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($message,) = @_;

    my $result = _store($self->{F_TMP}, $self->{F_INCOMING}, $message);
    $self->statistics->{enqueue_msg}++;
    return $result;
}

=item dequeue() [PUBLIC]

  Usage     : my $result = $instance->dequeue();
  Purpose   : Dequeue next message from the INCOMING message queue
  Arguments : None
  Returns   : ARRAY REFERENCE on SUCCESS, UNDEF on FAILURE
                STRING $queue_id - item queue ID
                STRING $file_name - item file name
  Throws    : Dies, if not invoked on object instance or if given invalid arguments

=cut

sub dequeue()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);

    my $result;
    my $next;
    
    while (1) {
        # Initialize result
        $result = undef;
        
        # Fetch next item from the queue
        $next = $self->_next();
        
        if ($next) {
            eval {
                # Attempt to move the file to the PENDING queue
                $result = _move($next->[1], $self->{F_PENDING});
                $self->statistics->{dequeue}++;
            };
            last if $result;
        }
        else {
            last;
        }
    }
    return $result;
}

#-------------------------------------------------------------------------------

=item banish($) [PUBLIC]

  Usage     : my $id = $instance->banish($queue_id);
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Dies, if not invoked on object instance or if given invalid arguments

=cut

sub banish($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($queue_id,) = @_;
    confess "Queue item ID must be given as first argument" unless $queue_id;

    # Check if the given item exists within the pending queue folder
    my $queue_file = $self->{F_PENDING}.'/'.$queue_id;
    return undef unless (-f $queue_file);

    $self->statistics->{banish}++;
    
    # Move the file to ERROR folder
    return _move($queue_file, $self->{F_ERROR});
}

=item commit($;$) [PUBLIC]

  Usage     : my $result = $instance->commit($queue_id);
  Purpose   : Commit message with given identifier in PENDING queue
  Arguments :
  Returns   :
  Throws    : Dies, if not invoked on object instance or if given invalid arguments

=cut

sub commit($;$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($queue_id, $message) = @_;
    confess "Queue item ID must be given as first argument" unless $queue_id;
    
    # Check if the given queue ID exists
    my $queue_file = $self->{F_PENDING}.'/'.$queue_id;
    return undef unless (-f $queue_file);

    $self->statistics->{commit}++;

    # Remove the message permanently
    unless ($self->{F_DONE}) {
        $queue_file = _remove($queue_file);
        return [$queue_id, $queue_file];
    }
    # Replace the original message with new one
    elsif ($message) {
        $queue_file = _remove($queue_file);
        return _store($self->{F_TMP}, $self->{F_DONE}, $message, $queue_id);
    }
    # Move the message to DONE folder
    else {
        return _move($queue_file, $self->{F_DONE}, $queue_id);
    }
}

=item remove($) [PUBLIC]

  Usage     : my $id = $instance->remove($queue_id);
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Dies, if not invoked on object instance or if given invalid arguments

=cut

sub remove($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($queue_id,) = @_;
    confess "Queue item ID must be given as first argument" unless $queue_id;

    # Check if the given item exists within the pending queue folder
    my $queue_file = $self->{F_PENDING}.'/'.$queue_id;
    return undef unless (-f $queue_file);

    $self->statistics->{remove}++;
    
    # Remove the file from PENDING folder
    return _remove($queue_file);
}
#-------------------------------------------------------------------------------

=item count_incoming(;$) [PUBLIC]

  Usage     : my $count = $instance->count_incoming();
  Purpose   : Get number of message queue records in both message queues
  Arguments : BOOL $real - don't count empty records (slower) [OPTIONAL]
  Returns   : INTEGER $count
  Throws    : Dies, if not invoked on object instance or if given invalid arguments

=cut

sub count_incoming(;$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($real,) = @_;

    return scalar $self->_fetch_files($self->{F_INCOMING}, $self->{LIMIT}) || 0 unless $real;
    return scalar $self->_fetch_files($self->{F_INCOMING}) || 0;
}

=item count_pending(;$) [PUBLIC]

  Usage     : my $count = $instance->count_pending();
  Purpose   : Get number of message queue records in both message queues
  Arguments : BOOL $real - don't count empty records (slower) [OPTIONAL]
  Returns   : INTEGER $count
  Throws    : Dies, if not invoked on object instance or if given invalid arguments

=cut

sub count_pending(;$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($real,) = @_;

    return scalar $self->_fetch_files($self->{F_PENDING}, $self->{LIMIT}) || 0 unless $real;
    return scalar $self->_fetch_files($self->{F_PENDING}) || 0;
}

=item count_done(;$) [PUBLIC]

  Usage     : my $count = $instance->count_done();
  Purpose   : Get number of message queue records in both message queues
  Arguments : BOOL $real - don't count empty records (slower) [OPTIONAL]
  Returns   : INTEGER $count
  Throws    : Dies, if not invoked on object instance or if given invalid arguments

=cut

sub count_done(;$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($real,) = @_;

    return 0 unless $self->{F_DONE};
    return scalar $self->_fetch_files($self->{F_DONE}, $self->{LIMIT}) || 0 unless $real;
    return scalar $self->_fetch_files($self->{F_DONE}) || 0;
}

=item is_incoming($) [PUBLIC]

  Usage     : my $result = $instance->exists($message_id);
  Purpose   : Check, if the message wit given ID is already present in the queue
  Arguments : INTEGER $message_id - message identifier [MANDATORY]
  Returns   : BOOL $result
  Throws    : Dies, if not invoked on object instance or if given invalid arguments

=cut

sub is_incoming($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($queue_id,) = @_;

    # Check if the given queue ID exists
    return -f $self->{F_INCOMING}.'/'.$queue_id;
}

=item is_pending($) [PUBLIC]

  Usage     : my $result = $instance->exists($message_id);
  Purpose   : Check, if the message wit given ID is already present in the queue
  Arguments : INTEGER $message_id - message identifier [MANDATORY]
  Returns   : BOOL $result
  Throws    : Dies, if not invoked on object instance or if given invalid arguments

=cut

sub is_pending($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($queue_id,) = @_;

    # Check if the given queue ID exists
    return -f $self->{F_PENDING}.'/'.$queue_id;
}

=item statistics() [PUBLIC]

  Usage     : my $statistics = $instance->statistics();
  Purpose   : Get processing statistics
  Arguments : None
  Returns   : HASH REFERENCE $statistics with various keys
  Throws    : Dies, if not invoked on object instance or if given invalid arguments
  Comments  : Mainly for debugging, development and optimization purposes

=cut

sub statistics()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    return $self->{STATISTICS};
}

#-------------------------------------------------------------------------------
# Private interface
#-------------------------------------------------------------------------------

# _init [PROTECTED]
#
#  Usage     : return $self->_init(@_);
#  Purpose   : Initialize the new Mentat::Processor::Queue::Folder; instance
#  Arguments : Unknown see the description in appropriate subclass
#  Returns   : Mentat::Processor::Queue::Folder; $self reference
#  Throws    : Dies, if not invoked on object instance or if given invalid arguments
#  Comments  : This method must never be called directly, it is supposed to be overloaded by subclasses.

sub _init
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    ### Initializing Mentat.Processor.Queue.Folder instance...
    my %config = @_;

    confess "Constructor expects configurations as argument" unless %config;
    die "Configuration 'incoming_folder' is missing\n" unless $config{incoming_folder};
    die "Configuration 'pending_folder' is missing\n"  unless $config{pending_folder};
    die "Configuration 'temp_folder' is missing\n"     unless $config{temp_folder};
    die "Configuration 'error_folder' is missing\n"    unless $config{error_folder};

    # Folder for incoming messages
    $self->{F_INCOMING} = $config{incoming_folder};
    # Folder for pending messsages
    $self->{F_PENDING} = $config{pending_folder};
    # Folder for done messages
    $self->{F_DONE} = $config{done_folder};
    # Temporary folder
    $self->{F_TMP} = $config{temp_folder};
    # Error folder
    $self->{F_ERROR} = $config{error_folder};

    # Queue of incoming messages
    $self->{Q_INCOMING} = [];

    # Message fetch limit
    $self->{LIMIT} = ($config{fetch_limit}) ? int($config{fetch_limit}) : FETCH_LIMIT();

    # Processing statistics
    $self->{STATISTICS} = {};

    return $self;
}

sub _fetch_files($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($dir, $limit) = @_;

    my @files = ();

    my ($dir_handle, $file);
    opendir($dir_handle, $dir) or die "Can't open folder '$dir': $!";
    while (defined($file = readdir($dir_handle))) {
        # Only plain files
        next unless (-f "$dir/$file");
        # Return only limited amount of entries
        last if ($limit and (scalar @files >= $limit));

        push(@files, [$file, "$dir/$file"]);
    }
    closedir($dir_handle);

    #return _sort_st(@files);
    return @files;
}

sub _next()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);

    # Refresh the contents of the INCOMING queue, if necessary
    unless (scalar @{$self->{Q_INCOMING}}) {
        $self->{Q_INCOMING} = [$self->_fetch_files($self->{F_INCOMING}, $self->{LIMIT})];
    }
    # Skip nonexisting items
    while ($self->{Q_INCOMING}->[0]) {
        # File exists, stop
        last if -f $self->{Q_INCOMING}->[0]->[1];
        # File does not exist, pop and retry
        shift @{$self->{Q_INCOMING}};
    }
    return shift @{$self->{Q_INCOMING}};
}

#-------------------------------------------------------------------------------

sub _move($$;$$)
{
    my ($file, $dir, $queue_id) = @_;
    confess "File name must be given as argument" unless $file;
    confess "File '$file' is not ordinary file" unless -f $file;
    #confess "File '$file' is not writable" unless -w $file;
    confess "Target folder name must be given as argument" unless $dir;
    confess "Target folder '$dir' is not directory" unless -d $dir;
    #confess "Target folder '$dir' is not writable" unless -w $dir;

    my $dfile;
    $queue_id = basename($file) unless $queue_id;
    $dfile = $dir.'/'.$queue_id;

    # Handle the situation, when the destination file already exists
    if (-f $dfile) {
        my $cnt = 0;
        while (-f $dir.'/'.(++$cnt)."_".$queue_id) { next; }
        $queue_id = $cnt."_".$queue_id;
        $dfile = $dir.'/'.$queue_id;
    }

    $file  = $1 if ($file  =~ /(.*)/);
    $dfile = $1 if ($dfile =~ /(.*)/);

    print STDERR "[Q:$queue_id]: Moving $file => $dfile\n" if $DEVEL;
    move($file, $dfile) or confess "Unable to move file '$file' to '$dfile': $!";
    return [$queue_id, $dfile];
}

sub _store($$$;$)
{
    my ($tmp, $dir, $message, $queue_id) = @_;
    confess "Temporary folder name must be given as argument" unless $dir;
    confess "Temporary folder '$dir' is not directory" unless -d $dir;
    #confess "Temporary folder '$dir' is not writable" unless -w $dir;
    confess "Target folder name must be given as argument" unless $dir;
    confess "Target folder '$dir' is not directory" unless -d $dir;
    #confess "Target folder '$dir' is not writable" unless -w $dir;
    confess "Message must be given as argument" unless $message;

    # Store the message within the directory
    my ($id, $tfile) = $message->to_dir($tmp);

    $queue_id = ($queue_id)?$queue_id:$id;

    my $dfile = $dir.'/'.$queue_id;

    $tfile = $1 if ($tfile =~ /(.*)/);
    $dfile = $1 if ($dfile =~ /(.*)/);

    print STDERR "[Q:$queue_id]: Storing message to $dfile\n" if $DEVEL;
    move($tfile, $dfile) or confess "Unable to move file '$tfile' to '$dfile': $!";
    return [$queue_id, $dfile];
}

sub _remove($)
{
    my ($file,) = @_;
    confess "File name must be given as argument" unless $file;
    confess "File '$file' is not ordinary file" unless -f $file;
    #confess "File '$file' is not writable" unless -w $file;

    $file  = $1 if ($file  =~ /(.*)/);

    print STDERR "[Q]: Removing $file permanently\n" if $DEVEL;
    unlink($file) or confess "Unable to remove file '$file'";
    return $file;
}

#-------------------------------------------------------------------------------

# The Schwartzian Transform sort
sub _sort_st(@)
{
    #return () unless scalar @_;

    # Algorithm resource: http://raleigh.pm.org/sorting.html
    # @_ exists, and contains filenames
    #return map { $_->[0] }              # restore original values
    #       sort { $b->[1] <=> $a->[1] } # sort
    #       map { [$_, -M $_->[1]] } @_; # transform: value, sortkey
}

# The "Packed Default" Sort
sub _sort_pd(@)
{
    # Algorithm resource: http://raleigh.pm.org/sorting.html
    # @_ exists, and contains filenames
    #return map substr($_, 4) =>
    #       sort
    #       map { pack(`C4', /(\d+)\. (\d+)\. (\d+)\. (\d+)/) . $_ } => @a;
}

=pod

=back

=cut

1;
