package Mentat::Processor::Scheduler;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Mentat::Processor::Scheduler - Short module description

=head1 SYNOPSIS

 use Mentat::Processor::Scheduler;

=head1 DESCRIPTION

Extensive module description

=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.

=head1 SEE ALSO

perl(1), Parent::Module(3).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';

#use Data::Dumper; #-+-> DEVEL ONLY <-+-#
#use Smart::Comments; #-+-> DEVEL ONLY <-+-#

#-- Perl CPAN modules ---------------------------------------------------------#

#-- Custom application modules ------------------------------------------------#

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL @ISA);
    $VERSION = '0.1';
    $DEVEL   = 0;
}


#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

=item new() [PUBLIC,STATIC]

  Usage     : my $instance = Mentat::Processor::Scheduler->new()
  Purpose   : Create and return reference to the new instance
  Arguments : Same as the the _init() method
  Returns   : Mentat::Processor::Scheduler reference on success, croaks on failure
  Throws    : Croaks, if not invoked on class
  Comments  : Internally uses _init() method, subclasses should never overload this method.
  See Also  : _init() method

=cut

sub new
{
    my $class = shift;
    confess "Class method not invoked on class" if blessed($class);
    my $self = bless ({}, $class);
    return $self->_init(@_);
}

=item schedule($;@) [ABSTRACT,PUBLIC]

  Usage     : my $id = $instance->schedule($event, @args);
  Purpose   : Schedule new event at the end of event queue
  Arguments : STRING $event - event name [MANDATORY]
              MIXED @arguments - event arguments [OPTIONAL]
  Returns   : MIXED $id unique event identifier
  Throws    : Croaks, if not invoked on class

=cut

sub schedule($;@)
{
    my $self = shift;
    confess "Method needs to be implemented in subclass";
}

=item schedule_first($;@) [ABSTRACT,PUBLIC]

  Usage     : my $id = $instance->schedule_first($event, @args);
  Purpose   : Schedule new event at the beginning of event queue
  Arguments : STRING $event - event name [MANDATORY]
              MIXED @arguments - event arguments [OPTIONAL]
  Returns   : MIXED $id unique event identifier
  Throws    : Croaks, if not invoked on class

=cut

sub schedule_first($;@)
{
    my $self = shift;
    confess "Method needs to be implemented in subclass";
}

=item schedule_at($$;@) [ABSTRACT,PUBLIC]

  Usage     : my $id = $instance->schedule_at($event, $timestamp, @args);
  Purpose   : Schedule new event at specific time
  Arguments : STRING $event - event name [MANDATORY]
              INTEGER $timestamp - unix timestamp [MANDATORY]
              MIXED @arguments - event arguments [OPTIONAL]
  Returns   : MIXED $id unique event identifier
  Throws    : Croaks, if not invoked on class

=cut

sub schedule_at($$;@)
{
    my $self = shift;
    confess "Method needs to be implemented in subclass";
}

=item next_event() [PUBLIC]

  Usage     : my $event = $instance->next_event();
  Purpose   : Fetch next event from the event queue
  Arguments : None
  Returns   : ARRAY REFERENCE $event on SUCCESS, UNDEF on FAILURE
  Throws    : Croaks, if not invoked on instance

=cut

sub next_event()
{
    my $self = shift;
    confess "Method needs to be implemented in subclass";
}

=item cancel_event() [ABSTRACT,PUBLIC]

  Usage     : my $error = $instance->cancel_event($id);
  Purpose   : Cancel event with given identifier
  Arguments : MIXED $id - event identifier
  Returns   : STRING $error on FAILURE, EMPTY STRING on SUCCESS
  Throws    : Croaks, if not invoked on class

=cut

sub cancel_event()
{
    my $self = shift;
    confess "Method needs to be implemented in subclass";
}

=item count() [ABSTRACT,PUBLIC]

  Usage     : my $count = $instance->count();
  Purpose   : Get number of events in event queue
  Arguments : None
  Returns   : INTEGER $count
  Throws    : Croaks, if not invoked on class
  Comments  : Mainly for debugging, development and optimization purposes

=cut

sub count()
{
    my $self = shift;
    confess "Method needs to be implemented in subclass";
}


=item statistics() [ABSTRACT,PUBLIC]

  Usage     : my $statistics = $instance->statistics();
  Purpose   : Get processing statistics
  Arguments : None
  Returns   : HASH REFERENCE $statistics with various keys
  Throws    : Croaks, if not invoked on class
  Comments  : Mainly for debugging, development and optimization purposes

=cut

sub statistics()
{
    my $self = shift;
    confess "Method needs to be implemented in subclass";
}

#-------------------------------------------------------------------------------
# Private interface
#-------------------------------------------------------------------------------

# _init [PROTECTED]
#
#  Usage     : return $self->_init(@_);
#  Purpose   : Initialize the new Mentat::Processor::Scheduler instance
#  Arguments : Unknown
#  Returns   : Mentat::Processor::Scheduler reference
#  Throws    : Croaks, if not invoked on instance
#  Comments  : This method must never be called directly, it is supposed to be overloaded by subclasses.

sub _init
{
    my $self = shift;
    confess "Method needs to be implemented in subclass";
}

# _id() [PROTECTED]
#
#  Usage     : my $id = $instance->_id();
#  Purpose   : Generate unique event identifier
#  Arguments : None
#  Returns   : INTEGER $id
#  Throws    : Croaks, if not invoked on instance

sub _id()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    return ++$self->{ID_GEN};
}

=pod

=back

=cut

1;
